@@ -46,7 +46,7 @@ LOG = logging.getLogger(__name__)
 _FILESYSTEM_CONFIGS = [
     cfg.StrOpt('filesystem_store_datadir',
                default='/var/lib/glance/images',
-               help=_("""
+               help="""
 Directory to which the filesystem backend store writes images.
 
 Upon start up, Glance creates the directory if it doesn't already
@@ -69,9 +69,9 @@ Related options:
     * ``filesystem_store_datadirs``
     * ``filesystem_store_file_perm``
 
-""")),
+"""),
     cfg.MultiStrOpt('filesystem_store_datadirs',
-                    help=_("""
+                    help="""
 List of directories and their priorities to which the filesystem
 backend store writes images.
 
@@ -88,7 +88,7 @@ defaults to zero.
 
 More information on configuring filesystem store with multiple store
 directories can be found at
-http://docs.openstack.org/developer/glance/configuring.html
+https://docs.openstack.org/glance/latest/configuration/configuring.html
 
 NOTE: This directory is used only when filesystem store is used as a
 storage backend. Either ``filesystem_store_datadir`` or
@@ -105,9 +105,9 @@ Related options:
     * ``filesystem_store_datadir``
     * ``filesystem_store_file_perm``
 
-""")),
+"""),
     cfg.StrOpt('filesystem_store_metadata_file',
-               help=_("""
+               help="""
 Filesystem store metadata file.
 
 The path to a file which contains the metadata to be returned with
@@ -122,10 +122,10 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.IntOpt('filesystem_store_file_perm',
                default=0,
-               help=_("""
+               help="""
 File access permissions for the image files.
 
 Set the intended file access permissions for image data. This provides
@@ -138,7 +138,7 @@ to the  default permissions. This value will be decoded as an octal
 digit.
 
 For more information, please refer the documentation at
-http://docs.openstack.org/developer/glance/configuring.html
+https://docs.openstack.org/glance/latest/configuration/configuring.html
 
 Possible values:
     * A valid file access permission
@@ -148,7 +148,7 @@ Possible values:
 Related options:
     * None
 
-"""))]
+""")]
 
 MULTI_FILESYSTEM_METADATA_SCHEMA = {
     "type": "array",
@@ -180,7 +180,7 @@ class StoreLocation(glance_store.location.StoreLocation):
         versions of Python.
         """
         pieces = urllib.parse.urlparse(uri)
-        assert pieces.scheme in ('file', 'filesystem')
+        self.validate_schemas(uri, valid_schemas=('file://', 'filesystem://'))
         self.scheme = pieces.scheme
         path = (pieces.netloc + pieces.path).strip()
         if path == '':
@@ -273,13 +273,18 @@ class Store(glance_store.driver.Store):
         :datadir is a directory path in which glance writes image files.
         """
 
-        if self.conf.glance_store.filesystem_store_file_perm <= 0:
+        if self.backend_group:
+            fstore_perm = getattr(
+                self.conf, self.backend_group).filesystem_store_file_perm
+        else:
+            fstore_perm = self.conf.glance_store.filesystem_store_file_perm
+
+        if fstore_perm <= 0:
             return
 
         try:
             mode = os.stat(datadir)[stat.ST_MODE]
-            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
-                       8)
+            perm = int(str(fstore_perm), 8)
             if perm & stat.S_IRWXO > 0:
                 if not mode & stat.S_IXOTH:
                     # chmod o+x
@@ -378,26 +383,37 @@ class Store(glance_store.driver.Store):
         this method. If the store was not able to successfully configure
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
-        if not (self.conf.glance_store.filesystem_store_datadir or
-                self.conf.glance_store.filesystem_store_datadirs):
+        if self.backend_group:
+            fdir = getattr(
+                self.conf, self.backend_group).filesystem_store_datadir
+            fdirs = getattr(
+                self.conf, self.backend_group).filesystem_store_datadirs
+            fstore_perm = getattr(
+                self.conf, self.backend_group).filesystem_store_file_perm
+            meta_file = getattr(
+                self.conf, self.backend_group).filesystem_store_metadata_file
+        else:
+            fdir = self.conf.glance_store.filesystem_store_datadir
+            fdirs = self.conf.glance_store.filesystem_store_datadirs
+            fstore_perm = self.conf.glance_store.filesystem_store_file_perm
+            meta_file = self.conf.glance_store.filesystem_store_metadata_file
+
+        if not (fdir or fdirs):
             reason = (_("Specify at least 'filesystem_store_datadir' or "
                         "'filesystem_store_datadirs' option"))
             LOG.error(reason)
             raise exceptions.BadStoreConfiguration(store_name="filesystem",
                                                    reason=reason)
 
-        if (self.conf.glance_store.filesystem_store_datadir and
-                self.conf.glance_store.filesystem_store_datadirs):
-
+        if fdir and fdirs:
             reason = (_("Specify either 'filesystem_store_datadir' or "
                         "'filesystem_store_datadirs' option"))
             LOG.error(reason)
             raise exceptions.BadStoreConfiguration(store_name="filesystem",
                                                    reason=reason)
 
-        if self.conf.glance_store.filesystem_store_file_perm > 0:
-            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
-                       8)
+        if fstore_perm > 0:
+            perm = int(str(fstore_perm), 8)
             if not perm & stat.S_IRUSR:
                 reason = _LE("Specified an invalid "
                              "'filesystem_store_file_perm' option which "
@@ -410,13 +426,13 @@ class Store(glance_store.driver.Store):
 
         self.multiple_datadirs = False
         directory_paths = set()
-        if self.conf.glance_store.filesystem_store_datadir:
-            self.datadir = self.conf.glance_store.filesystem_store_datadir
+        if fdir:
+            self.datadir = fdir
             directory_paths.add(self.datadir)
         else:
             self.multiple_datadirs = True
             self.priority_data_map = {}
-            for datadir in self.conf.glance_store.filesystem_store_datadirs:
+            for datadir in fdirs:
                 (datadir_path,
                  priority) = self._get_datadir_path_and_priority(datadir)
                 priority_paths = self.priority_data_map.setdefault(
@@ -431,9 +447,8 @@ class Store(glance_store.driver.Store):
 
         self._create_image_directories(directory_paths)
 
-        metadata_file = self.conf.glance_store.filesystem_store_metadata_file
-        if metadata_file:
-            self._validate_metadata(metadata_file)
+        if meta_file:
+            self._validate_metadata(meta_file)
 
     def _check_directory_paths(self, datadir_path, directory_paths,
                                priority_paths):
@@ -644,8 +659,9 @@ class Store(glance_store.driver.Store):
 
         return best_datadir
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -655,12 +671,15 @@ class Store(glance_store.driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: The request context
         :param verifier: An object used to verify signatures for images
 
-        :retval: tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
 
         :note:: By default, the backend writes the image data to a file
               `/<DATADIR>/<ID>`, where <DATADIR> is the value of
@@ -673,7 +692,7 @@ class Store(glance_store.driver.Store):
 
         if os.path.exists(filepath):
             raise exceptions.Duplicate(image=filepath)
-
+        os_hash_value = hashlib.new(str(hashing_algo))
         checksum = hashlib.md5()
         bytes_written = 0
         try:
@@ -681,6 +700,7 @@ class Store(glance_store.driver.Store):
                 for buf in utils.chunkreadable(image_file,
                                                self.WRITE_CHUNKSIZE):
                     bytes_written += len(buf)
+                    os_hash_value.update(buf)
                     checksum.update(buf)
                     if verifier:
                         verifier.update(buf)
@@ -696,25 +716,40 @@ class Store(glance_store.driver.Store):
             with excutils.save_and_reraise_exception():
                 self._delete_partial(filepath, image_id)
 
+        hash_hex = os_hash_value.hexdigest()
         checksum_hex = checksum.hexdigest()
         metadata = self._get_metadata(filepath)
 
-        LOG.debug(_("Wrote %(bytes_written)d bytes to %(filepath)s with "
-                    "checksum %(checksum_hex)s"),
+        LOG.debug(("Wrote %(bytes_written)d bytes to %(filepath)s with "
+                   "checksum %(checksum_hex)s and multihash %(hash_hex)s"),
                   {'bytes_written': bytes_written,
                    'filepath': filepath,
-                   'checksum_hex': checksum_hex})
+                   'checksum_hex': checksum_hex,
+                   'hash_hex': hash_hex})
 
-        if self.conf.glance_store.filesystem_store_file_perm > 0:
-            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
-                       8)
+        if self.backend_group:
+            fstore_perm = getattr(
+                self.conf, self.backend_group).filesystem_store_file_perm
+        else:
+            fstore_perm = self.conf.glance_store.filesystem_store_file_perm
+
+        if fstore_perm > 0:
+            perm = int(str(fstore_perm), 8)
             try:
                 os.chmod(filepath, perm)
             except (IOError, OSError):
                 LOG.warning(_LW("Unable to set permission to image: %s") %
                             filepath)
 
-        return ('file://%s' % filepath, bytes_written, checksum_hex, metadata)
+        # Add store backend information to location metadata
+        if self.backend_group:
+            metadata['backend'] = u"%s" % self.backend_group
+
+        return ('file://%s' % filepath,
+                bytes_written,
+                checksum_hex,
+                hash_hex,
+                metadata)
 
     @staticmethod
     def _delete_partial(filepath, iid):
